def example(extract='m', estimate=False):
    import pandas as pd
    import geopandas as gpd
    import larch
    larch.__version__

    import larch.exampville

    larch.exampville.files.shapefile

    taz_shape = gpd.read_file("zip://"+larch.exampville.files.shapefile)

    larch.exampville.files.employment

    emp = pd.read_csv(larch.exampville.files.employment, index_col='TAZ')

    emp.head()

    larch.exampville.files.skims

    skims = larch.OMX( larch.exampville.files.skims, mode='r' )
    skims

    larch.exampville.files.hh

    hh = pd.read_csv( larch.exampville.files.hh )

    hh.head()

    larch.exampville.files.person

    pp = pd.read_csv( larch.exampville.files.person )

    pp.head()

    larch.exampville.files.tour

    tour = pd.read_csv( larch.exampville.files.tour )

    tour.head()
    if not estimate:
        if isinstance(extract, str):
            return locals()[extract]
        else:
            _locals = locals()
            return [_locals.get(i) for i in extract]

    if isinstance(extract, str):
        return locals()[extract]
    else:
        _locals = locals()
        return [_locals.get(i) for i in extract]
