
# Example code extracted automatically from Larch documentation.
# Do not edit these files manually.
from ._000_mtc_data import example as _000_mtc_data
from ._001_mnl import example as _001_mnl
from ._002_mtc import example as _002_mtc
from ._017A_mnl_segmented import example as _017A_mnl_segmented
from ._017_mnl_final import example as _017_mnl_final
from ._022_nl import example as _022_nl
from ._028_nl import example as _028_nl
from ._101_swissmetro_mnl import example as _101_swissmetro_mnl
from ._102_swissmetro_weighted import example as _102_swissmetro_weighted
from ._109_swissmetro_nl import example as _109_swissmetro_nl
from ._200_exampville import example as _200_exampville
from ._201_exville_mode_choice import example as _201_exville_mode_choice
from ._202_exville_mc_logsums import example as _202_exville_mc_logsums
from ._203_exville_dest import example as _203_exville_dest
from ._300_itinerary import example as _300_itinerary
from ._301_itin_mnl import example as _301_itin_mnl

ex = {}
ex[0] = _000_mtc_data
ex[1] = _001_mnl
ex[2] = _002_mtc
ex['017A'] = _017A_mnl_segmented
ex[17] = _017_mnl_final
ex[22] = _022_nl
ex[28] = _028_nl
ex[101] = _101_swissmetro_mnl
ex[102] = _102_swissmetro_weighted
ex[109] = _109_swissmetro_nl
ex[200] = _200_exampville
ex[201] = _201_exville_mode_choice
ex[202] = _202_exville_mc_logsums
ex[203] = _203_exville_dest
ex[300] = _300_itinerary
ex[301] = _301_itin_mnl
