def example(extract='m', estimate=False):
    import gzip, os, pandas as pd
    import larch.numba as lx

    itin = pd.read_csv(lx.example_file("arc"), index_col=['id_case','id_alt'])
    itin.info()

    d = lx.Dataset.from_idca(itin, crack=True)
    d
    if not estimate:
        if isinstance(extract, str):
            return locals()[extract]
        else:
            _locals = locals()
            return [_locals.get(i) for i in extract]

    if isinstance(extract, str):
        return locals()[extract]
    else:
        _locals = locals()
        return [_locals.get(i) for i in extract]
